#include "population.h"
#include <random>
#include <iostream>

Population::Population(const std::string& name, int size, double vaccination_rate, bool patient_zero)
    : name(name) {
    int vaccinated_count = static_cast<int>(size * vaccination_rate);
    int current_vaccinated = 0;
    for (int i = 0; i < size; ++i) {
        if (current_vaccinated < vaccinated_count) {
            people.emplace_back(Person::Vaccinated);
            current_vaccinated++;
        } else {
            people.emplace_back(Person::Susceptible);
        }
    }
    if (patient_zero && size > 0) {
        people[vaccinated_count].setState(Person::Infectious);
    }
}

void Population::simulateDay(double transmissibility, int disease_duration) {
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(0, people.size() - 1);

    for (auto& person : people) {
        if (person.getState() == Person::Infectious) {
            for (int i = 0; i < 6; ++i) {
                int contact_index = dis(gen);
                people[contact_index].tryInfect(transmissibility);
            }
            person.updateState(disease_duration);
        }
    }
}

void Population::randomContact(Population& other_population, double transmissibility) {
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis_this(0, people.size() - 1);
    std::uniform_int_distribution<> dis_other(0, other_population.people.size() - 1);

    for (int i = 0; i < 5; ++i) {
        int contact_this = dis_this(gen);
        int contact_other = dis_other(gen);
        people[contact_this].tryInfect(transmissibility);
        other_population.people[contact_other].tryInfect(transmissibility);
    }
}

bool Population::hasInfectious() const {
    for (const auto& person : people) {
        if (person.getState() == Person::Infectious) {
            return true;
        }
    }
    return false;
}

std::string Population::getName() const {
    return name;
}

void Population::getStatistics(int& susceptible, int& infectious, int& vaccinated, int& recovered) const {
    susceptible = infectious = vaccinated = recovered = 0;
    for (const auto& person : people) {
        switch (person.getState()) {
            case Person::Susceptible: susceptible++; break;
            case Person::Infectious: infectious++; break;
            case Person::Vaccinated: vaccinated++; break;
            case Person::Recovered: recovered++; break;
        }
    }
}
