#include "simulation.h"
#include <iostream>
#include <fstream>
#include <random>
#include <filesystem>

Simulation::Simulation(const ConfigParser& config)
    : simulation_runs(config.getSimulationRuns()),
      disease_transmissibility(config.getDiseaseTransmissibility()),
      disease_duration(config.getDiseaseDuration()),
      total_steps(0) {
    
    const INIReader& reader = config.getReader();
    for (int i = 1; i <= config.getNumPopulations(); ++i) {
        std::string section = "population_" + std::to_string(i);
        std::string name = reader.Get(section, "name", "default");
        int size = reader.GetInteger(section, "size", 100);
        double vaccination_rate = reader.GetReal(section, "vaccination_rate", 0.0);
        bool patient_zero = reader.GetBoolean(section, "patient_0", false);
        populations.emplace_back(name, size, vaccination_rate, patient_zero);
    }
}

void Simulation::run() {
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(0, populations.size() - 1);

    bool has_infectious = true;

    // Create results directory if it doesn't exist
    if (!std::filesystem::exists("results")) {
        std::filesystem::create_directory("results");
    }

    // Open the detailed results file and write the header
    std::ofstream details_file("results/disease_details.csv");
    if (!details_file.is_open()) {
        std::cerr << "Error opening disease_details.csv for writing" << std::endl;
        return;
    }
    details_file << "day,population,infectious,recovered,susceptible,vaccinated\n";

    while (has_infectious) {
        has_infectious = false;
        for (auto& population : populations) {
            population.simulateDay(disease_transmissibility, disease_duration);
            if (population.hasInfectious()) {
                has_infectious = true;
            }
        }
        total_steps++;

        int pop1 = dis(gen);
        int pop2 = dis(gen);
        if (pop1 != pop2) {
            populations[pop1].randomContact(populations[pop2], disease_transmissibility);
        }

        for (const auto& population : populations) {
            int susceptible, infectious, vaccinated, recovered;
            population.getStatistics(susceptible, infectious, vaccinated, recovered);
            details_file << total_steps << "," << population.getName() << "," << infectious << "," << recovered << "," << susceptible << "," << vaccinated << "\n";
        }
    }
    details_file.close();
}

void Simulation::outputResults() {
    std::ofstream stats_file("results/disease_stats.csv");
    if (!stats_file.is_open()) {
        std::cerr << "Error opening disease_stats.csv for writing" << std::endl;
        return;
    }
    stats_file << "key,value\n";

    int total_susceptible = 0;
    int total_infectious = 0;
    int total_vaccinated = 0;
    int total_recovered = 0;

    for (const auto& population : populations) {
        int susceptible, infectious, vaccinated, recovered;
        population.getStatistics(susceptible, infectious, vaccinated, recovered);
        total_susceptible += susceptible;
        total_infectious += infectious;
        total_vaccinated += vaccinated;
        total_recovered += recovered;
    }

    stats_file << "total_steps," << total_steps << "\n";
    stats_file << "susceptible_persons," << total_susceptible << "\n";
    stats_file << "infectious_persons," << total_infectious << "\n";
    stats_file << "vaccinated_persons," << total_vaccinated << "\n";
    stats_file << "recovered_persons," << total_recovered << "\n";
    stats_file.close();

    std::cout << "Simulation completed.\n";
    std::cout << "Total steps: " << total_steps << "\n";
    std::cout << "Susceptible: " << total_susceptible << "\n";
    std::cout << "Infectious: " << total_infectious << "\n";
    std::cout << "Vaccinated: " << total_vaccinated << "\n";
    std::cout << "Recovered: " << total_recovered << "\n";
}
