#define DOCTEST_CONFIG_IMPLEMENT_WITH_MAIN
#include "../include/doctest.h"
#include "../simulation/simulation.h"
#include "../simulation/config_parser.h"
#include <fstream>
#include <filesystem>

// Test Person class
TEST_CASE("Person state transitions") {
    Person person(Person::Susceptible);
    
    // Test initial state
    CHECK(person.getState() == Person::Susceptible);
    
    // Test infection attempt
    person.tryInfect(1.0);  // 100% transmissibility
    CHECK(person.getState() == Person::Infectious);
    
    // Test state update
    for (int i = 0; i < 3; ++i) {
        person.updateState(3);  // Disease duration is 3
    }
    CHECK(person.getState() == Person::Recovered);
}

// Test Population class
TEST_CASE("Population initialization and simulation") {
    Population population("TestPopulation", 100, 0.2, true);  // 20% vaccinated, patient zero

    // Check initial statistics
    int susceptible, infectious, vaccinated, recovered;
    population.getStatistics(susceptible, infectious, vaccinated, recovered);

    // Debug output to trace the issue
    std::cout << "Initial statistics - Susceptible: " << susceptible 
              << ", Infectious: " << infectious 
              << ", Vaccinated: " << vaccinated 
              << ", Recovered: " << recovered << std::endl;

    CHECK(susceptible == 79);
    CHECK(infectious == 1);  // Patient zero
    CHECK(vaccinated == 20);
    CHECK(recovered == 0);

    // Simulate a day
    population.simulateDay(1.0, 3);  // 100% transmissibility, disease duration 3
    population.getStatistics(susceptible, infectious, vaccinated, recovered);

    // Debug output after simulation
    std::cout << "After one day - Susceptible: " << susceptible 
              << ", Infectious: " << infectious 
              << ", Vaccinated: " << vaccinated 
              << ", Recovered: " << recovered << std::endl;

    CHECK(susceptible < 79);  // Some susceptible persons should be infected
    CHECK(infectious > 1);    // More than one person should be infectious
}

// Test Simulation class
TEST_CASE("Simulation run and results") {
    ConfigParser config("disease_in.ini");
    Simulation simulation(config);
    
    // Run the simulation
    simulation.run();

    // Check if results files are created
    CHECK(std::filesystem::exists("results/disease_stats.csv"));
    CHECK(std::filesystem::exists("results/disease_details.csv"));

    // Validate the summary statistics file
    std::ifstream stats_file("results/disease_stats.csv");
    std::string line;
    std::getline(stats_file, line);  // Skip the header
    std::getline(stats_file, line);  // Read the total_steps line
    CHECK(line.find("total_steps") != std::string::npos);
}